@@ -33,7 +33,8 @@ arch_check_usable_kernel () {
 
 arch_get_kernel () {
 	CPUS=`grep 'ncpus probed' "$CPUINFO" | cut -d: -f2`
-	if [ "$CPUS" -ne 1 ]; then
+	TYPE=`grep '^type' "$CPUINFO" | head -n1 | cut -d: -f2 | sed -e 's/^[[:space:]]//'`
+	if [ "$CPUS" -ne 1 ] || [ "$TYPE" = "sun4v" ]; then
 		if [ "$1" = sparc32 ] && [ "$KERNEL_MAJOR" = 2.6 ]; then
 			# No working SMP yet
 			:
