@@ -517,7 +517,7 @@ set_disk_named(const char *name, PedDisk *disk)
         devices[index].disk = disk;
 }
 
-/* True iff the partition doesn't exist on the storage device */
+/* True if the partition doesn't exist on the storage device */
 bool
 named_partition_is_virtual(const char *name, PedSector start, PedSector end)
 {
@@ -714,8 +714,58 @@ resize_partition(PedDisk *disk, PedPartition *part,
                 }
                 if (NULL == fs && NULL != ped_file_system_probe(&(part->geom)))
                         return false;
+                if (NULL != fs)
+                        constraint = ped_file_system_get_resize_constraint(fs);
+                else
+                        constraint = ped_constraint_any(disk->dev);
         } else {
+                PedFileSystemType *fs_type;
+                PedGeometry *fs_geom;
+                PedAlignment start_align;
+                PedGeometry full_dev;
                 fs = NULL;
+                fs_type = ped_file_system_probe(&(part->geom));
+                log("probed file system: %s", NULL != fs_type ? "yes" : "no");
+                if (NULL != fs_type)
+                        fs_geom = ped_file_system_probe_specific(fs_type,
+                                                                 &part->geom);
+                else
+                        fs_geom = NULL;
+                if (NULL != fs_geom && (fs_geom->start < (part->geom).start
+                                        || fs_geom->end > (part->geom).end)) {
+                        log("broken filesystem detected");
+                        ped_geometry_destroy(fs_geom);
+                        fs_geom = NULL;
+                }
+                if (NULL == fs_geom && NULL != fs_type)
+                        return false;
+                if (NULL != fs_geom) {
+                        /* We cannot resize or move the fs but we can
+                         * move the end of the partition so long as it
+                         * contains the whole fs.
+                         */
+                        if (ped_alignment_init(&start_align, fs_geom->start, 0)
+                            && ped_geometry_init(&full_dev, disk->dev,
+                                                 0, disk->dev->length - 1)) {
+                                constraint = ped_constraint_new(
+                                        &start_align,
+                                        ped_alignment_any,
+                                        &full_dev, &full_dev,
+                                        fs_geom->length,
+                                        disk->dev->length);
+                        } else {
+                                constraint = NULL;
+                        }
+                        ped_geometry_destroy(fs_geom);
+                } else {
+                        constraint = ped_constraint_any(disk->dev);
+                }
+        }
+        if (NULL == constraint) {
+                log("failed to get resize constraint");
+                if (NULL != fs)
+                        ped_file_system_close(fs);
+                return false;
         }
         log("try to check the file system for errors");
         if (NULL != fs && !timered_file_system_check(fs)) {
@@ -727,10 +777,6 @@ resize_partition(PedDisk *disk, PedPartition *part,
         log("successfully checked");
         if (part->type & PED_PARTITION_LOGICAL)
                 maximize_extended_partition(disk);
-        if (NULL != fs)
-                constraint = ped_file_system_get_resize_constraint(fs);
-        else
-                constraint = ped_constraint_any(disk->dev);
         if (!ped_disk_set_partition_geom(disk, part, constraint, start, end))
                 result = false;
         else if (NULL == fs)
@@ -1496,6 +1542,24 @@ command_set_name()
         activate_exception_handler();
 }
 
+void
+command_get_max_primary()
+{
+        scan_device_name();
+        if (dev == NULL)
+                critical_error("The device %s is not opened.", device_name);
+        log("command_get_max_primary()");
+        open_out();
+        oprintf("OK\n");
+        deactivate_exception_handler();
+        if (disk != NULL && disk->type != NULL)
+                oprintf("%d\n",
+                        ped_disk_get_max_primary_partition_count(disk));
+        else
+                oprintf("\n");
+        activate_exception_handler();
+}
+
 void
 command_uses_extended()
 {
@@ -1586,7 +1650,6 @@ command_change_file_system()
         }
         free(id);
         fstype = ped_file_system_type_get(s_fstype);
-        free(s_fstype);
         if (fstype == NULL) {
                 log("Filesystem %s not found, let's see if it is a flag",
                     s_fstype);
@@ -1600,6 +1663,7 @@ command_change_file_system()
         } else {
                 ped_partition_set_system(part, fstype);
         }
+        free(s_fstype);
         oprintf("OK\n");
 }
 
@@ -1654,7 +1718,6 @@ command_create_file_system()
                 critical_error("No such partition: %s", id);
         free(id);
         fstype = ped_file_system_type_get(s_fstype);
-        free(s_fstype);
         if (fstype == NULL)
                 critical_error("Bad file system type: %s", s_fstype);
         ped_partition_set_system(part, fstype);
@@ -1664,6 +1727,7 @@ command_create_file_system()
                 ped_disk_commit_to_dev(disk);
         }
         activate_exception_handler();
+        free(s_fstype);
         oprintf("OK\n");
         if (fs != NULL)
                 oprintf("OK\n");
@@ -2062,11 +2126,34 @@ command_get_disk_type()
         activate_exception_handler();
 }
 
+void
+command_is_busy()
+{
+        char *id;
+        PedPartition *part;
+        log("command_is_busy()");
+        scan_device_name();
+        if (dev == NULL)
+                critical_error("The device %s is not opened.", device_name);
+        open_out();
+        if (1 != iscanf("%as", &id))
+                critical_error("Expected partition id");
+        log("command_is_busy: busy check for id %s", id);
+        part = partition_with_id(disk, id);
+        oprintf("OK\n");
+        if (ped_partition_is_busy(part)) {
+                oprintf("yes\n");
+        } else {
+                oprintf("no\n");
+        }
+        free(id);
+}
+
 void
 make_fifo(char* name)
 {
     int status;
-    status = mkfifo(name, 0x644);
+    status = mkfifo(name, 0644);
     if ((status != 0))
             if (errno != EEXIST) {
                     perror("Cannot create FIFO");
@@ -2201,6 +2288,8 @@ main_loop()
                         command_set_name();
                 else if (!strcasecmp(str, "USES_NAMES"))
                         command_uses_names();
+                else if (!strcasecmp(str, "GET_MAX_PRIMARY"))
+                        command_get_max_primary();
                 else if (!strcasecmp(str, "USES_EXTENDED"))
                         command_uses_extended();
                 else if (!strcasecmp(str, "FILE_SYSTEM_TYPES"))
@@ -2232,6 +2321,8 @@ main_loop()
                         command_copy_partition();
                 else if (!strcasecmp(str, "GET_DISK_TYPE"))
                         command_get_disk_type();
+                else if (!strcasecmp(str, "IS_BUSY"))
+                        command_is_busy();
                 else
                         critical_error("Unknown command %s", str);
                 free(str);
@@ -2252,6 +2343,8 @@ main(int argc, char *argv[])
                 close(i);
 
         // Set up signal handling
+        memset(&act,0,sizeof(struct sigaction));
+        memset(&oldact,0,sizeof(struct sigaction));
         act.sa_handler = prnt_sig_hdlr;
         sigemptyset(&act.sa_mask);
 
