@@ -3,6 +3,7 @@
 
 # Copyright (C) 2005 Javier Carranza and others for Guadalinex
 # Copyright (C) 2005, 2006 Canonical Ltd.
+# Copyright (C) 2007 Mario Limonciello
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -42,6 +43,7 @@ from apt.progress import FetchProgress, InstallProgress
 sys.path.insert(0, '/usr/lib/ubiquity')
 
 from ubiquity import misc
+from ubiquity import osextras
 from ubiquity.components import language_apply, apt_setup, timezone_apply, \
                                 clock_setup, console_setup_apply, \
                                 usersetup_apply, hw_detect, check_kernels, \
@@ -175,8 +177,15 @@ class DebconfInstallProgress(InstallProgress):
 
         self.statusfd.close()
 
-        # Redirect stdout to stderr to avoid it interfering with our
-        # debconf protocol stream.
+        # Redirect stdin from /dev/null and stdout to stderr to avoid them
+        # interfering with our debconf protocol stream.
+        saved_stdin = os.dup(0)
+        try:
+            null = os.open('/dev/null', os.O_RDONLY)
+            os.dup2(null, 0)
+            os.close(null)
+        except OSError:
+            pass
         saved_stdout = os.dup(1)
         os.dup2(2, 1)
 
@@ -212,7 +221,9 @@ class DebconfInstallProgress(InstallProgress):
                 except OSError:
                     break
 
-            # Put back stdout.
+            # Put back stdin and stdout.
+            os.dup2(saved_stdin, 0)
+            os.close(saved_stdin)
             os.dup2(saved_stdout, 1)
             os.close(saved_stdout)
 
@@ -391,7 +402,12 @@ class Install:
             self.configure_bootloader()
 
             self.db.progress('SET', 94)
-            self.db.progress('REGION', 94, 99)
+            self.db.progress('REGION', 94, 95)
+            self.db.progress('INFO', 'ubiquity/install/installing')
+            self.install_extras()
+
+            self.db.progress('SET', 95)
+            self.db.progress('REGION', 95, 99)
             self.db.progress('INFO', 'ubiquity/install/removing')
             self.remove_extras()
 
@@ -487,7 +503,17 @@ class Install:
                 os.mknod(targetpath, stat.S_IFSOCK | mode)
             elif stat.S_ISREG(st.st_mode):
                 if not os.path.exists(targetpath):
-                    shutil.copyfile(sourcepath, targetpath)
+                    sourcefh = None
+                    targetfh = None
+                    try:
+                        sourcefh = open(sourcepath, 'rb')
+                        targetfh = open(targetpath, 'wb')
+                        shutil.copyfileobj(sourcefh, targetfh)
+                    finally:
+                        if targetfh:
+                            targetfh.close()
+                        if sourcefh:
+                            sourcefh.close()
 
             copied_size += st.st_size
             os.lchown(targetpath, st.st_uid, st.st_gid)
@@ -545,10 +571,10 @@ class Install:
             os.makedirs(target_dir)
 
         for log_file in ('/var/log/syslog', '/var/log/partman',
-                         '/var/log/installer/version'):
+                         '/var/log/installer/version', '/var/log/casper.log'):
             target_log_file = os.path.join(target_dir,
                                            os.path.basename(log_file))
-            if not misc.ex('cp', '-a', log_file, target_log_file):
+            if not misc.execute('cp', '-a', log_file, target_log_file):
                 syslog.syslog(syslog.LOG_ERR,
                               'Failed to copy installation log file')
             os.chmod(target_log_file, stat.S_IRUSR | stat.S_IWUSR)
@@ -565,6 +591,15 @@ class Install:
             status.close()
         except IOError:
             pass
+        try:
+            if self.db.get('oem-config/enable') == 'true':
+                oem_id = self.db.get('oem-config/id')
+                oem_id_file = open(
+                    os.path.join(self.target, 'var/log/installer/oem-id'))
+                print >>oem_id_file, oem_id
+                oem_id_file.close()
+        except (debconf.DebconfError, IOError):
+            pass
 
 
     def mount_one_image(self, fsfile, mountpoint=None):
@@ -603,12 +638,12 @@ class Install:
         if dev == '':
             raise InstallStepError("No loop device available for %s" % fsfile)
 
-        misc.ex('losetup', dev, fsfile)
+        misc.execute('losetup', dev, fsfile)
         if mountpoint is None:
             mountpoint = '/var/lib/ubiquity/%s' % sysloop
         if not os.path.isdir(mountpoint):
             os.mkdir(mountpoint)
-        misc.ex('mount', dev, mountpoint)
+        misc.execute('mount', dev, mountpoint)
 
         return (dev, mountpoint)
 
@@ -630,7 +665,7 @@ class Install:
             for line in mounts:
                 (device, fstype) = line.split()[1:3]
                 if fstype == 'squashfs' and os.path.exists(device):
-                    misc.ex('mount', '--bind', device, self.source)
+                    misc.execute('mount', '--bind', device, self.source)
                     self.mountpoints.append(self.source)
                     mounts.close()
                     return
@@ -671,9 +706,10 @@ class Install:
             assert self.devs
             assert self.mountpoints
 
-            misc.ex('mount', '-t', 'unionfs', '-o',
-                    'dirs=' + map(lambda x: '%s=ro' % x, self.mountpoints),
-                    'unionfs', self.source)
+            misc.execute('mount', '-t', 'unionfs', '-o',
+                         'dirs=' + ':'.join(map(lambda x: '%s=ro' % x,
+                                                self.mountpoints)),
+                         'unionfs', self.source)
             self.mountpoints.append(self.source)
 
     def umount_source(self):
@@ -685,10 +721,10 @@ class Install:
         mountpoints.reverse()
 
         for mountpoint in mountpoints:
-            if not misc.ex('umount', mountpoint):
+            if not misc.execute('umount', mountpoint):
                 raise InstallStepError("Failed to unmount %s" % mountpoint)
         for dev in devs:
-            if dev != '' and not misc.ex('losetup', '-d', dev):
+            if dev != '' and not misc.execute('losetup', '-d', dev):
                 raise InstallStepError(
                     "Failed to detach loopback device %s" % dev)
 
@@ -772,14 +808,60 @@ exit 0"""
     def configure_apt(self):
         """Configure /etc/apt/sources.list."""
 
+        # TODO cjwatson 2007-07-06: Much of the following is
+        # cloned-and-hacked from base-installer/debian/postinst. Perhaps we
+        # should come up with a way to avoid this.
+
+        # Make apt trust CDs. This is not on by default (we think).
+        # This will be left in place on the installed system.
+        apt_conf_tc = open(os.path.join(
+            self.target, 'etc/apt/apt.conf.d/00trustcdrom'), 'w')
+        print >>apt_conf_tc, 'APT::Authentication::TrustCDROM "true";'
+        apt_conf_tc.close()
+
         # Avoid clock skew causing gpg verification issues.
         # This file will be left in place until the end of the install.
         apt_conf_itc = open(os.path.join(
             self.target, 'etc/apt/apt.conf.d/00IgnoreTimeConflict'), 'w')
-        print >>apt_conf_itc, ('Acquire::gpgv::Options {'
-                               ' "--ignore-time-conflict"; };')
+        print >>apt_conf_itc, \
+            'Acquire::gpgv::Options { "--ignore-time-conflict"; };'
         apt_conf_itc.close()
 
+        try:
+            if self.db.get('debian-installer/allow_unauthenticated') == 'true':
+                apt_conf_au = open(
+                    os.path.join(self.target,
+                                 'etc/apt/apt.conf.d/00AllowUnauthenticated'),
+                    'w')
+                print >>apt_conf_au, 'APT::Get::AllowUnauthenticated "true";'
+                print >>apt_conf_au, \
+                    'Aptitude::CmdLine::Ignore-Trust-Violations "true";'
+                apt_conf_au.close()
+        except debconf.DebconfError:
+            pass
+
+        # let apt inside the chroot see the cdrom
+        target_cdrom = os.path.join(self.target, 'cdrom')
+        misc.execute('umount', target_cdrom)
+        if not os.path.exists(target_cdrom):
+            os.mkdir(target_cdrom)
+        misc.execute('mount', '--bind', '/cdrom', target_cdrom)
+
+        # Make apt-cdrom and apt not unmount/mount CD-ROMs.
+        # This file will be left in place until the end of the install.
+        apt_conf_nmc = open(os.path.join(
+            self.target, 'etc/apt/apt.conf.d/00NoMountCDROM'), 'w')
+        print >>apt_conf_nmc, textwrap.dedent("""\
+            APT::CDROM::NoMount "true";
+            Acquire::cdrom {
+              mount "/cdrom";
+              "/cdrom/" {
+                Mount  "true";
+                UMount "true";
+              };
+            }""")
+        apt_conf_nmc.close()
+
         dbfilter = apt_setup.AptSetup(None)
         ret = dbfilter.run_command(auto_process=True)
         if ret != 0:
@@ -809,6 +891,16 @@ exit 0"""
         record.close()
 
 
+    def query_recorded_installed(self):
+        apt_installed = set()
+        if os.path.exists("/var/lib/ubiquity/apt-installed"):
+            record_file = open("/var/lib/ubiquity/apt-installed")
+            for line in record_file:
+                apt_installed.add(line.strip())
+            record_file.close()
+        return apt_installed
+
+
     def mark_install(self, cache, pkg):
         cachedpkg = self.get_cache_pkg(cache, pkg)
         if cachedpkg is not None and not cachedpkg.isInstalled:
@@ -867,86 +959,8 @@ exit 0"""
                 to_install.append(pattern.replace('$LL', lp))
             # More extensive language support packages.
             to_install.append('language-support-%s' % lp)
-        self.record_installed(to_install)
 
-        self.db.progress('START', 0, 100, 'ubiquity/langpacks/title')
-
-        self.db.progress('REGION', 0, 10)
-        fetchprogress = DebconfFetchProgress(
-            self.db, 'ubiquity/langpacks/title',
-            'ubiquity/install/apt_indices_starting',
-            'ubiquity/install/apt_indices')
-        cache = Cache()
-
-        if cache._depcache.BrokenCount > 0:
-            syslog.syslog(
-                'not installing language packs, since there are broken '
-                'packages: %s' % ', '.join(self.broken_packages(cache)))
-            self.db.progress('STOP')
-            return
-
-        try:
-            # update() returns False on failure and 0 on success. Madness!
-            if cache.update(fetchprogress) not in (0, True):
-                fetchprogress.stop()
-                self.db.progress('STOP')
-                return
-        except IOError, e:
-            for line in str(e).split('\n'):
-                syslog.syslog(syslog.LOG_WARNING, line)
-            self.db.progress('STOP')
-            raise
-        cache.open(None)
-        self.db.progress('SET', 10)
-
-        self.db.progress('REGION', 10, 100)
-        fetchprogress = DebconfFetchProgress(
-            self.db, 'ubiquity/langpacks/title', None,
-            'ubiquity/langpacks/packages')
-        installprogress = DebconfInstallProgress(
-            self.db, 'ubiquity/langpacks/title', 'ubiquity/install/apt_info')
-
-        for lp in to_install:
-            self.mark_install(cache, lp)
-        installed_pkgs = []
-        for pkg in cache.keys():
-            if (cache[pkg].markedInstall or cache[pkg].markedUpgrade or
-                cache[pkg].markedReinstall or cache[pkg].markedDowngrade):
-                installed_pkgs.append(pkg)
-        self.record_installed(installed_pkgs)
-
-        commit_error = None
-        try:
-            if not cache.commit(fetchprogress, installprogress):
-                fetchprogress.stop()
-                installprogress.finishUpdate()
-                self.db.progress('STOP')
-                return
-        except IOError, e:
-            for line in str(e).split('\n'):
-                syslog.syslog(syslog.LOG_WARNING, line)
-            commit_error = str(e)
-        except SystemError, e:
-            for line in str(e).split('\n'):
-                syslog.syslog(syslog.LOG_WARNING, line)
-            commit_error = str(e)
-        self.db.progress('SET', 100)
-
-        cache.open(None)
-        if commit_error or cache._depcache.BrokenCount > 0:
-            if commit_error is None:
-                commit_error = ''
-            brokenpkgs = self.broken_packages(cache)
-            syslog.syslog('broken packages after language pack installation: '
-                          '%s' % ', '.join(brokenpkgs))
-            self.db.subst('ubiquity/install/broken_install', 'ERROR',
-                          commit_error)
-            self.db.subst('ubiquity/install/broken_install', 'PACKAGES',
-                          ', '.join(brokenpkgs))
-            self.db.input('critical', 'ubiquity/install/broken_install')
-            self.db.go()
-
-        self.db.progress('STOP')
+        self.do_install(to_install, langpacks=True)
 
 
     def configure_timezone(self):
@@ -982,7 +996,7 @@ exit 0"""
         ret = dbfilter.run_command(auto_process=True)
         if ret != 0:
             raise InstallStepError("UserSetupApply failed with code %d" % ret)
-    
+
     def configure_ma(self):
         """import documents, settings, and users from previous operating
         systems."""
@@ -1025,8 +1039,8 @@ exit 0"""
 
         self.db.progress('INFO', 'ubiquity/install/hardware')
 
-        misc.ex('/usr/lib/ubiquity/debian-installer-utils'
-                '/register-module.post-base-installer')
+        misc.execute('/usr/lib/ubiquity/debian-installer-utils'
+                     '/register-module.post-base-installer')
 
         resume = self.get_resume_partition()
         if resume is not None:
@@ -1069,6 +1083,16 @@ exit 0"""
         except debconf.DebconfError:
             pass
 
+        try:
+            os.unlink('/target/etc/papersize')
+        except OSError:
+            pass
+        self.chrex('ucf', '--purge', '/etc/papersize')
+        try:
+            self.set_debconf('libpaper/defaultpaper', '')
+        except debconf.DebconfError:
+            pass
+
         self.chroot_setup()
         self.chrex('dpkg-divert', '--package', 'ubiquity', '--rename',
                    '--quiet', '--add', '/usr/sbin/update-initramfs')
@@ -1080,7 +1104,8 @@ exit 0"""
         packages = ['linux-image-' + self.kernel_version,
                     'linux-restricted-modules-' + self.kernel_version,
                     'usplash',
-                    'popularity-contest']
+                    'popularity-contest',
+                    'libpaper1']
 
         try:
             for package in packages:
@@ -1160,11 +1185,11 @@ exit 0"""
 
     def configure_network(self):
         """Automatically configure the network.
-        
+
         At present, the only thing the user gets to tweak in the UI is the
         hostname. Some other things will be copied from the live filesystem,
         so changes made there will be reflected in the installed system.
-        
+
         Unfortunately, at present we have to duplicate a fair bit of netcfg
         here, because it's hard to drive netcfg in a way that won't try to
         bring interfaces up and down."""
@@ -1210,95 +1235,102 @@ exit 0"""
             ff02::3 ip6-allhosts""")
         hosts.close()
 
-        # TODO cjwatson 2006-03-30: from <bits/ioctls.h>; ugh, but no
-        # binding available
-        SIOCGIFHWADDR = 0x8927
-        # <net/if_arp.h>
-        ARPHRD_ETHER = 1
+        persistent_net = '/etc/udev/rules.d/70-persistent-net.rules'
+        if os.path.exists(persistent_net):
+            shutil.copy2(persistent_net,
+                         os.path.join(self.target, persistent_net[1:]))
+        else:
+            # TODO cjwatson 2006-03-30: from <bits/ioctls.h>; ugh, but no
+            # binding available
+            SIOCGIFHWADDR = 0x8927
+            # <net/if_arp.h>
+            ARPHRD_ETHER = 1
 
-        if_names = {}
-        sock = socket.socket(socket.SOCK_DGRAM)
-        interfaces = self.get_all_interfaces()
-        for i in range(len(interfaces)):
-            if_names[interfaces[i]] = struct.unpack('H6s',
-                fcntl.ioctl(sock.fileno(), SIOCGIFHWADDR,
-                            struct.pack('256s', interfaces[i]))[16:24])
-        sock.close()
+            if_names = {}
+            sock = socket.socket(socket.SOCK_DGRAM)
+            interfaces = self.get_all_interfaces()
+            for i in range(len(interfaces)):
+                if_names[interfaces[i]] = struct.unpack('H6s',
+                    fcntl.ioctl(sock.fileno(), SIOCGIFHWADDR,
+                                struct.pack('256s', interfaces[i]))[16:24])
+            sock.close()
 
-        iftab = open(os.path.join(self.target, 'etc/iftab'), 'w')
+            iftab = open(os.path.join(self.target, 'etc/iftab'), 'w')
 
-        print >>iftab, textwrap.dedent("""\
-            # This file assigns persistent names to network interfaces.
-            # See iftab(5) for syntax.
-            """)
+            print >>iftab, textwrap.dedent("""\
+                # This file assigns persistent names to network interfaces.
+                # See iftab(5) for syntax.
+                """)
 
-        for i in range(len(interfaces)):
-            dup = False
-            with_arp = False
+            for i in range(len(interfaces)):
+                dup = False
+                with_arp = False
 
-            if_name = if_names[interfaces[i]]
-            if if_name is None or if_name[0] != ARPHRD_ETHER:
-                continue
-
-            for j in range(len(interfaces)):
-                if i == j or if_names[interfaces[j]] is None:
-                    continue
-                if if_name[1] != if_names[interfaces[j]][1]:
+                if_name = if_names[interfaces[i]]
+                if if_name is None or if_name[0] != ARPHRD_ETHER:
                     continue
 
-                if if_names[interfaces[j]][0] == ARPHRD_ETHER:
-                    dup = True
+                for j in range(len(interfaces)):
+                    if i == j or if_names[interfaces[j]] is None:
+                        continue
+                    if if_name[1] != if_names[interfaces[j]][1]:
+                        continue
 
-            if dup:
-                continue
+                    if if_names[interfaces[j]][0] == ARPHRD_ETHER:
+                        dup = True
 
-            line = (interfaces[i] + " mac " +
-                    ':'.join(['%02x' % ord(if_name[1][c]) for c in range(6)]))
-            line += " arp %d" % if_name[0]
-            print >>iftab, line
+                if dup:
+                    continue
 
-        iftab.close()
+                line = (interfaces[i] + " mac " +
+                        ':'.join(['%02x' % ord(if_name[1][c])
+                                  for c in range(6)]))
+                line += " arp %d" % if_name[0]
+                print >>iftab, line
+
+            iftab.close()
 
 
     def configure_bootloader(self):
         """configuring and installing boot loader into installed
         hardware system."""
+        install_bootloader = self.db.get('ubiquity/install_bootloader')
+        if install_bootloader == "true":
+            misc.execute('mount', '--bind', '/proc', self.target + '/proc')
+            misc.execute('mount', '--bind', '/dev', self.target + '/dev')
 
-        misc.ex('mount', '--bind', '/proc', self.target + '/proc')
-        misc.ex('mount', '--bind', '/dev', self.target + '/dev')
+            archdetect = subprocess.Popen(['archdetect'], stdout=subprocess.PIPE)
+            subarch = archdetect.communicate()[0].strip()
 
-        archdetect = subprocess.Popen(['archdetect'], stdout=subprocess.PIPE)
-        subarch = archdetect.communicate()[0].strip()
-
-        try:
-            if subarch.startswith('amd64/') or subarch.startswith('i386/'):
-                from ubiquity.components import grubinstaller
-                dbfilter = grubinstaller.GrubInstaller(None)
-                ret = dbfilter.run_command(auto_process=True)
-                if ret != 0:
-                    raise InstallStepError(
-                        "GrubInstaller failed with code %d" % ret)
-            elif subarch == 'powerpc/ps3':
-                from ubiquity.components import kbootinstaller
-                dbfilter = kbootinstaller.KbootInstaller(None)
-                ret = dbfilter.run_command(auto_process=True)
-                if ret != 0:
-                    raise InstallStepError(
-                        "KbootInstaller failed with code %d" % ret)
-            elif subarch.startswith('powerpc/'):
-                from ubiquity.components import yabootinstaller
-                dbfilter = yabootinstaller.YabootInstaller(None)
-                ret = dbfilter.run_command(auto_process=True)
-                if ret != 0:
-                    raise InstallStepError(
-                        "YabootInstaller failed with code %d" % ret)
-            else:
+            try:
+                if subarch.startswith('amd64/') or subarch.startswith('i386/'):
+                    from ubiquity.components import grubinstaller
+                    dbfilter = grubinstaller.GrubInstaller(None)
+                    ret = dbfilter.run_command(auto_process=True)
+                    if ret != 0:
+                        raise InstallStepError(
+                            "GrubInstaller failed with code %d" % ret)
+                elif subarch == 'powerpc/ps3':
+                    from ubiquity.components import kbootinstaller
+                    dbfilter = kbootinstaller.KbootInstaller(None)
+                    ret = dbfilter.run_command(auto_process=True)
+                    if ret != 0:
+                        raise InstallStepError(
+                            "KbootInstaller failed with code %d" % ret)
+                elif subarch.startswith('powerpc/'):
+                    from ubiquity.components import yabootinstaller
+                    dbfilter = yabootinstaller.YabootInstaller(None)
+                    ret = dbfilter.run_command(auto_process=True)
+                    if ret != 0:
+                        raise InstallStepError(
+                            "YabootInstaller failed with code %d" % ret)
+                else:
+                    raise InstallStepError("No bootloader installer found")
+            except ImportError:
                 raise InstallStepError("No bootloader installer found")
-        except ImportError:
-            raise InstallStepError("No bootloader installer found")
 
-        misc.ex('umount', '-f', self.target + '/proc')
-        misc.ex('umount', '-f', self.target + '/dev')
+            misc.execute('umount', '-f', self.target + '/proc')
+            misc.execute('umount', '-f', self.target + '/dev')
 
 
     def broken_packages(self, cache):
@@ -1312,6 +1344,93 @@ exit 0"""
                 continue
         return brokenpkgs
 
+    def do_install(self, to_install, langpacks=False):
+        if langpacks:
+            self.db.progress('START', 0, 10, 'ubiquity/langpacks/title')
+        else:
+            self.db.progress('START', 0, 10, 'ubiquity/install/title')
+        self.db.progress('INFO', 'ubiquity/install/find_installables')
+
+        if langpacks:
+            # ... otherwise we're installing packages that have already been
+            # recorded
+            self.record_installed(to_install)
+
+        self.db.progress('REGION', 0, 1)
+        fetchprogress = DebconfFetchProgress(
+            self.db, 'ubiquity/install/title',
+            'ubiquity/install/apt_indices_starting',
+            'ubiquity/install/apt_indices')
+        cache = Cache()
+
+        if cache._depcache.BrokenCount > 0:
+            syslog.syslog(
+                'not installing additional packages, since there are broken '
+                'packages: %s' % ', '.join(self.broken_packages(cache)))
+            self.db.progress('STOP')
+            return
+
+        for pkg in to_install:
+            self.mark_install(cache, pkg)
+
+        self.db.progress('SET', 1)
+        self.db.progress('REGION', 1, 10)
+        if langpacks:
+            fetchprogress = DebconfFetchProgress(
+                self.db, 'ubiquity/langpacks/title', None,
+                'ubiquity/langpacks/packages')
+            installprogress = DebconfInstallProgress(
+                self.db, 'ubiquity/langpacks/title',
+                'ubiquity/install/apt_info')
+        else:
+            fetchprogress = DebconfFetchProgress(
+                self.db, 'ubiquity/install/title', None,
+                'ubiquity/install/fetch_remove')
+            installprogress = DebconfInstallProgress(
+                self.db, 'ubiquity/install/title',
+                'ubiquity/install/apt_info',
+                'ubiquity/install/apt_error_install')
+        self.chroot_setup()
+        commit_error = None
+        try:
+            try:
+                if not cache.commit(fetchprogress, installprogress):
+                    fetchprogress.stop()
+                    installprogress.finishUpdate()
+                    self.db.progress('STOP')
+                    return
+            except IOError, e:
+                for line in str(e).split('\n'):
+                    syslog.syslog(syslog.LOG_ERR, line)
+                fetchprogress.stop()
+                installprogress.finishUpdate()
+                self.db.progress('STOP')
+                return
+            except SystemError, e:
+                for line in str(e).split('\n'):
+                    syslog.syslog(syslog.LOG_ERR, line)
+                commit_error = str(e)
+        finally:
+            self.chroot_cleanup()
+        self.db.progress('SET', 10)
+
+        cache.open(None)
+        if commit_error or cache._depcache.BrokenCount > 0:
+            if commit_error is None:
+                commit_error = ''
+            brokenpkgs = self.broken_packages(cache)
+            syslog.syslog('broken packages after installation: '
+                          '%s' % ', '.join(brokenpkgs))
+            self.db.subst('ubiquity/install/broken_install', 'ERROR',
+                          commit_error)
+            self.db.subst('ubiquity/install/broken_install', 'PACKAGES',
+                          ', '.join(brokenpkgs))
+            self.db.input('critical', 'ubiquity/install/broken_install')
+            self.db.go()
+
+        self.db.progress('STOP')
+
+
     def do_remove(self, to_remove, recursive=False):
         self.db.progress('START', 0, 5, 'ubiquity/install/title')
         self.db.progress('INFO', 'ubiquity/install/find_removables')
@@ -1457,6 +1576,88 @@ exit 0"""
         self.db.progress('STOP')
 
 
+    def install_extras(self):
+        """Try to install additional packages requested by installer
+        components."""
+
+        # We only ever install these packages from the CD.
+        sources_list = os.path.join(self.target, 'etc/apt/sources.list')
+        os.rename(sources_list, "%s.apt-setup" % sources_list)
+        old_sources = open("%s.apt-setup" % sources_list)
+        new_sources = open(sources_list, 'w')
+        found_cdrom = False
+        for line in old_sources:
+            if 'cdrom:' in line:
+                print >>new_sources, line,
+                found_cdrom = True
+        new_sources.close()
+        old_sources.close()
+        if not found_cdrom:
+            os.rename("%s.apt-setup" % sources_list, sources_list)
+
+        self.do_install(self.query_recorded_installed())
+
+        if found_cdrom:
+            os.rename("%s.apt-setup" % sources_list, sources_list)
+
+        # TODO cjwatson 2007-08-09: python reimplementation of
+        # oem-config/finish-install.d/07oem-config-user. This really needs
+        # to die in a great big chemical fire and call the same shell script
+        # instead.
+        try:
+            if self.db.get('oem-config/enable') == 'true':
+                if os.path.isdir(os.path.join(self.target, 'home/oem')):
+                    open(os.path.join(self.target, 'home/oem/.hwdb'),
+                         'w').close()
+
+                    for desktop_file in (
+                        'usr/share/applications/oem-config-prepare-gtk.desktop',
+                        'usr/share/applications/kde/oem-config-prepare-kde.desktop'):
+                        if os.path.exists(os.path.join(self.target,
+                                                       desktop_file)):
+                            desktop_base = os.path.basename(desktop_file)
+                            self.chrex('install', '-D',
+                                       '-o', 'oem', '-g', 'oem',
+                                       '/%s' % desktop_file,
+                                       '/home/oem/Desktop/%s' % desktop_base)
+                            break
+
+                # Some serious horribleness is needed here to handle absolute
+                # symlinks.
+                for name in ('gdm-cdd.conf', 'gdm.conf'):
+                    gdm_conf = osextras.realpath_root(
+                        self.target, os.path.join('/etc/gdm', name))
+                    if os.path.isfile(gdm_conf):
+                        self.chrex('sed', '-i.oem',
+                                   '-e', 's/^AutomaticLoginEnable=.*$/AutomaticLoginEnable=true/',
+                                   '-e', 's/^AutomaticLogin=.*$/AutomaticLogin=oem/',
+                                   '-e', 's/^TimedLoginEnable=.*$/TimedLoginEnable=true/',
+                                   '-e', 's/^TimedLogin=.*$/TimedLogin=oem/',
+                                   '-e', 's/^TimedLoginDelay=.*$/TimedLoginDelay=10/',
+                                   os.path.join('/etc/gdm', name));
+                        break
+
+                kdmrc = os.path.join(self.target, 'etc/kde3/kdm/kdmrc')
+                if os.path.isfile(kdmrc):
+                    misc.execute('sed', '-i.oem', '-r',
+                                 '-e', 's/^#?AutoLoginEnable=.*$/AutoLoginEnable=true/',
+                                 '-e', 's/^#?AutoLoginUser=.*$/AutoLoginUser=oem/',
+                                 '-e', 's/^#?AutoReLogin=.*$/AutoReLogin=true/',
+                                 kdmrc)
+
+                if osextras.find_on_path_root(self.target, 'kpersonalizer'):
+                    kpersonalizerrc = os.path.join(self.target,
+                                                   'etc/kde3/kpersonalizerrc')
+                    if not os.path.isfile(kpersonalizerrc):
+                        kpersonalizerrc_file = open(kpersonalizerrc, 'w')
+                        print >>kpersonalizerrc_file, '[General]'
+                        print >>kpersonalizerrc_file, 'FirstLogin=false'
+                        kpersonalizerrc_file.close()
+                        open('%s.created-by-oem', 'w').close()
+        except debconf.DebconfError:
+            pass
+
+
     def remove_extras(self):
         """Try to remove packages that are needed on the live CD but not on
         the installed system."""
@@ -1484,13 +1685,7 @@ exit 0"""
             difference = set()
 
         # Keep packages we explicitly installed.
-        apt_installed = set()
-        if os.path.exists("/var/lib/ubiquity/apt-installed"):
-            apt_installed_file = open("/var/lib/ubiquity/apt-installed")
-            for line in apt_installed_file:
-                apt_installed.add(line.strip())
-            apt_installed_file.close()
-        difference -= apt_installed
+        difference -= self.query_recorded_installed()
 
         if len(difference) == 0:
             return
@@ -1504,18 +1699,24 @@ exit 0"""
 
     def cleanup(self):
         """Miscellaneous cleanup tasks."""
-        try:
-            os.unlink(os.path.join(
-                self.target, 'etc/apt/apt.conf.d/00IgnoreTimeConflict'))
-        except:
-            pass
+
+        misc.execute('umount', os.path.join(self.target, 'cdrom'))
+
+        for apt_conf in ('00NoMountCDROM', '00IgnoreTimeConflict',
+                         '00AllowUnauthenticated'):
+            try:
+                os.unlink(os.path.join(
+                    self.target, 'etc/apt/apt.conf.d', apt_conf))
+            except:
+                pass
+
         if self.source == '/var/lib/ubiquity/source':
             self.umount_source()
 
 
     def chrex(self, *args):
         """executes commands on chroot system (provided by *args)."""
-        return misc.ex('chroot', self.target, *args)
+        return misc.execute('chroot', self.target, *args)
 
 
     def copy_debconf(self, package):
@@ -1527,9 +1728,9 @@ exit 0"""
         # preseeding in general, though.
         targetdb = os.path.join(self.target, 'var/cache/debconf/config.dat')
 
-        misc.ex('debconf-copydb', 'configdb', 'targetdb', '-p',
-                '^%s/' % package, '--config=Name:targetdb',
-                '--config=Driver:File','--config=Filename:' + targetdb)
+        misc.execute('debconf-copydb', 'configdb', 'targetdb', '-p',
+                     '^%s/' % package, '--config=Name:targetdb',
+                     '--config=Driver:File','--config=Filename:' + targetdb)
 
 
     def set_debconf(self, question, value):
@@ -1540,11 +1741,13 @@ exit 0"""
                                    '-fnoninteractive', 'ubiquity'],
                                   stdin=subprocess.PIPE,
                                   stdout=subprocess.PIPE, close_fds=True)
-        dc = debconf.Debconf(read=dccomm.stdout, write=dccomm.stdin)
-        dc.set(question, value)
-        dc.fset(question, 'seen', 'true')
-        dccomm.stdin.close()
-        dccomm.wait()
+        try:
+            dc = debconf.Debconf(read=dccomm.stdout, write=dccomm.stdin)
+            dc.set(question, value)
+            dc.fset(question, 'seen', 'true')
+        finally:
+            dccomm.stdin.close()
+            dccomm.wait()
 
 
     def reconfigure(self, package):
