@@ -24,6 +24,7 @@ import signal
 import errno
 import subprocess
 import re
+
 import debconf
 
 # Each widget should have a run(self, priority, question) method; this
@@ -227,6 +228,10 @@ class DebconfFilter:
             input_widgets = self.find_widgets([question])
 
             if len(input_widgets) > 0:
+                if 'UBIQUITY_AUTOMATIC' in os.environ:
+                    if self.db.fget(question, 'seen') == 'true':
+                        self.reply(30, 'question skipped', log=True)
+                        return True
                 self.debug('filter', 'widget found for', question)
                 if not input_widgets[0].run(priority, question):
                     self.debug('filter', 'widget requested backup')
