@@ -25,11 +25,13 @@ import signal
 import subprocess
 import re
 import syslog
+
 import debconf
 try:
     from debconf import DebconfCommunicator
 except ImportError:
     from ubiquity.debconfcommunicator import DebconfCommunicator
+
 from ubiquity.debconffilter import DebconfFilter
 
 # We identify as this to debconf.
@@ -189,6 +191,12 @@ class FilteredCommand(object):
 
         return call_again
 
+    def question_type(self, question):
+        try:
+            return self.db.metaget(question, 'Type')
+        except debconf.DebconfError:
+            return ''
+
     # Split a string on commas, stripping surrounding whitespace, and
     # honouring backslash-quoting.
     def split_choices(self, text):
@@ -293,6 +301,13 @@ class FilteredCommand(object):
         if seen:
             self.db.fset(name, 'seen', 'true')
 
+    def preseed_bool(self, name, value, seen=True):
+        if value:
+            self.preseed(name, 'true', seen, False)
+        else:
+            self.preseed(name, 'false', seen, False)
+
+
     def preseed_as_c(self, name, value, seen=True):
         self.preseed(name, self.translate_to_c(name, value), seen)
 
@@ -337,10 +352,20 @@ class FilteredCommand(object):
     # The confmodule asked a question; process it. Subclasses only need to
     # override this if they want to do something special like updating their
     # UI depending on what questions were asked.
+
+    # TODO: Make the steps references in the individual components, rather than
+    # having to define the relationship here?
     def run(self, priority, question):
+        if not self.frontend.installing:
+            # Make sure any started progress bars are stopped.
+            while self.frontend.progress_position.depth() != 0:
+                self.frontend.debconf_progress_stop()
+
         self.current_question = question
         if not self.done:
             self.succeeded = False
+            n = self.__class__.__name__
+            self.frontend.set_page(n)
             self.enter_ui_loop()
         return self.succeeded
 
